use std::sync::Arc;

use octopush_core::{
    builder::{builder_capabilities::BuilderCapabilities, DynBuilder},
    executor::{default_executor::DefaultExecutor, executor::DynExecutor},
    git::{github::GitHubGitProvider, DynGitProvider},
    schema::parser::{DefaultSchemaParser, DynSchemaParser},
    storage::{local::LocalStorageEngine, DynStorageEngine},
};

pub struct ServiceRegister {
    pub storage_engine: DynStorageEngine,
    pub git_provider: DynGitProvider,
    pub schema_parser: DynSchemaParser,
    pub builder: DynBuilder,
    pub executor: DynExecutor,
}

impl ServiceRegister {
    pub fn new() -> Self {
        let storage_engine = Arc::new(LocalStorageEngine::new("/tmp/octopush".into()));
        let git_provider = Arc::new(GitHubGitProvider::new(storage_engine.clone()));
        let schema_parser = Arc::new(DefaultSchemaParser::new());
        let builder = Arc::new(BuilderCapabilities::new());
        let executor = Arc::new(DefaultExecutor::new(builder.clone()));

        Self {
            storage_engine,
            git_provider,
            schema_parser,
            builder,
            executor,
        }
    }

    pub async fn cleanup(self) -> eyre::Result<()> {
        self.storage_engine.cleanup().await
    }
}
